@@ -2,13 +2,13 @@
 """Implement the Rock class and other core rocks functionality."""
 import datetime as dt
 import json
-from typing import List, Optional, Union
+from typing import List, Optional
+import warnings
 
 import numpy as np
 import pandas as pd
 import pydantic
 import rich
-from tqdm import tqdm
 
 import rocks
 
@@ -30,18 +30,39 @@ def ensure_list(value):
     return value
 
 
-def ensure_unique_taxonomy(value):
-    """If there are several taxonomic classifications in the ssoCard, pick the most recent one."""
-    if isinstance(value, list):
+def merge_entries(value):
+    """Turn list of dicts into dict of lists."""
 
-        value = {
-            "class": ", ".join(v["class"] for v in value),
-            "waverange": ", ".join(v["waverange"] for v in value),
-            "scheme": ", ".join(v["scheme"] for v in value),
-            "method": value[0]["method"],
-            "bibref": [v["bibref"] for v in value],
-        }
-    return value
+    if isinstance(value, list):  # taxonomy
+        pass
+    elif isinstance(value, dict):  # spin
+        value = list(value.values())
+
+    return_dict = {}
+
+    # Take keys of first dict
+    for key in value[0]:
+
+        # Turn all dicts in the list into lists
+        return_dict[key] = []
+
+        for v in value:
+            if key in v:
+                return_dict[key].append(v[key])
+            else:
+                # This entry of the current parameter does not have all
+                # values (eg "period" for Spin)
+
+                # Add the correct NaN value
+                if isinstance(value[0][key], dict):
+                    return_dict[key].append({})
+                elif isinstance(value[0][key], (float, int)):
+                    return_dict[key].append(np.nan)
+                elif isinstance(value[0][key], str):
+                    return_dict[key].append(None)
+    return return_dict
+
+    # return {key: [entry[key] for entry in value] for key in value[0]}
 
 
 # ------
@@ -49,48 +70,47 @@ def ensure_unique_taxonomy(value):
 
 # The lowest level in the ssoCard tree is the Value
 class Error(pydantic.BaseModel):
-    min_: Optional[float] = pydantic.Field(np.nan, alias="min")
-    max_: Optional[float] = pydantic.Field(np.nan, alias="max")
+    min_: float = pydantic.Field(np.nan, alias="min")
+    max_: float = pydantic.Field(np.nan, alias="max")
 
 
 class Value(pydantic.BaseModel):
     error: Error = Error(**{})
     value: Optional[float] = np.nan
+    path_unit: str = ""
 
     def __str__(self):
         """Print the value of a numerical parameter including
         its errors and its unit if available.
         """
 
-        unit = (
-            rocks.utils.get_unit(self.path_unit) if hasattr(self, "path_unit") else ""
-        )
+        unit = rocks.utils.get_unit(self.path_unit) if self.path_unit else ""
 
         if abs(self.error.min_) == abs(self.error.max_):
-            return f"{self.value} +- {self.error.max_} {unit}"
+            return f"{self.value:.4} +- {self.error.max_:.4} {unit}"
         else:
-            return f"{self.value} +- ({self.error.max_}, {self.error.min_}) {unit}"
+            return f"{self.value:.4} +- ({self.error.max_:.4}, {self.error.min_:.4}) {unit}"
 
 
-# The second lowest level is the Property
-class Property(pydantic.BaseModel):
+# The second lowest level is the Parameter
+class Parameter(pydantic.BaseModel):
     def __str__(self):
         return json.dumps(json.loads(self.json()), indent=2, sort_keys=True)
 
 
 # Other common branches are method and bibref
-class Method(Property):
+class Method(Parameter):
     doi: Optional[str] = ""
     name: Optional[str] = ""
-    year: Optional[int] = np.nan
+    year: Optional[int] = None
     title: Optional[str] = ""
     bibcode: Optional[str] = ""
     shortbib: Optional[str] = ""
 
 
-class Bibref(Property):
+class Bibref(Parameter):
     doi: Optional[str] = ""
-    year: Optional[int] = np.nan
+    year: Optional[int] = None
     title: Optional[str] = ""
     bibcode: Optional[str] = ""
     shortbib: Optional[str] = ""
@@ -98,7 +118,7 @@ class Bibref(Property):
 
 # ------
 # Dynamical parameters
-class OrbitalElements(Property):
+class OrbitalElements(Parameter):
     ceu: Value = Value(**{})
     author: Optional[str] = ""
     bibref: List[Bibref] = [Bibref(**{})]
@@ -106,21 +126,17 @@ class OrbitalElements(Property):
     ref_epoch: Optional[float] = np.nan
     inclination: Value = Value(**{})
     mean_motion: Value = Value(**{})
-    orbital_arc: Optional[int] = np.nan
+    orbital_arc: Optional[int] = None
     eccentricity: Value = Value(**{})
     mean_anomaly: Value = Value(**{})
     node_longitude: Value = Value(**{})
     orbital_period: Value = Value(**{})
     semi_major_axis: Value = Value(**{})
-    number_observation: Optional[int] = np.nan
+    number_observation: Optional[int] = None
     perihelion_argument: Value = Value(**{})
 
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", allow_reuse=True, pre=True
-    )(ensure_list)
 
-
-class ProperElements(Property):
+class ProperElements(Parameter):
     bibref: List[Bibref] = [Bibref(**{})]
     proper_g: Value = Value(**{})
     proper_s: Value = Value(**{})
@@ -129,67 +145,40 @@ class ProperElements(Property):
     proper_semi_major_axis: Value = Value(**{})
     proper_sine_inclination: Value = Value(**{})
 
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", allow_reuse=True, pre=True
-    )(ensure_list)
 
-    def __str__(self):
-        return self.json()
-
-
-class Family(Property):
+class Family(Parameter):
     bibref: List[Bibref] = [Bibref(**{})]
     family_name: Optional[str] = ""
-    family_number: Optional[int] = np.nan
+    family_number: Optional[int] = None
     family_status: Optional[str] = ""
 
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", allow_reuse=True, pre=True
-    )(ensure_list)
 
-    def __str__(self):
-        return self.json()
-
-
-class PairMembers(Property):
+class PairMembers(Parameter):
     sibling_name: Optional[str] = ""
     pair_delta_v: Optional[float] = np.nan
     pair_delta_a: Optional[float] = np.nan
     pair_delta_e: Optional[float] = np.nan
     pair_delta_i: Optional[float] = np.nan
-    sibling_number: Optional[int] = np.nan
+    sibling_number: Optional[int] = None
 
 
-class Pair(Property):
+class Pair(Parameter):
     members: List[PairMembers] = [PairMembers(**{})]
     bibref: List[Bibref] = [Bibref(**{})]
 
-    _ensure_list: classmethod = pydantic.validator(
-        "members", "bibref", allow_reuse=True, pre=True
-    )(ensure_list)
 
-    def __str__(self):
-        return self.json()
-
-
-class Yarkovsky(Property):
+class Yarkovsky(Parameter):
     S: Optional[float] = np.nan
     A2: Value = Value(**{})
     snr: Optional[float] = np.nan
     dadt: Value = Value(**{})
     bibref: List[Bibref] = [Bibref(**{})]
 
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", allow_reuse=True, pre=True
-    )(ensure_list)
-
     def __str__(self):
-        print_A2 = print_parameter(self.A2, "unit.dynamical.yarkovsky.A2.value")
-        print_dadt = print_parameter(self.dadt, "unit.dynamical.yarkovsky.dadt.value")
-        return "\n".join([print_A2, print_dadt])
+        return "\n".join([self.A2.__str__(), self.dadt.__str__()])
 
 
-class DynamicalParameters(Property):
+class DynamicalParameters(Parameter):
     pair: Pair = Pair(**{})
     family: Family = Family(**{})
     yarkovsky: Yarkovsky = Yarkovsky(**{})
@@ -223,7 +212,7 @@ class Color(Value):
     id_filter_2: Optional[str] = ""
 
 
-class Colors(Property):
+class Colors(Parameter):
     # Atlas
     c_o: List[Color] = [pydantic.Field(Color(**{}), alias="c-o")]
     # 2MASS / VISTA
@@ -231,34 +220,33 @@ class Colors(Property):
     J_K: List[Color] = [pydantic.Field(Color(**{}), alias="J-K")]
     H_K: List[Color] = [pydantic.Field(Color(**{}), alias="H-K")]
 
-    _ensure_list: classmethod = pydantic.validator("*", allow_reuse=True, pre=True)(
-        ensure_list
-    )
 
-
-class Diameter(Value):
+class Density(Value):
     method: List[Method] = []
     bibref: List[Bibref] = []
 
-    path_unit: str = "unit.physical.diameter.diameter"
+    path_unit: str = "unit.physical.density.value"
 
     _ensure_list: classmethod = pydantic.validator(
         "bibref", "method", allow_reuse=True, pre=True
     )(ensure_list)
 
 
+class Diameter(Value):
+    method: List[Method] = [Method(**{})]
+    bibref: List[Bibref] = [Bibref(**{})]
+
+    path_unit: str = "unit.physical.diameter.value"
+
+
 class Mass(Value):
     bibref: List[Bibref] = [Bibref(**{})]
     method: List[Method] = [Method(**{})]
 
-    path_unit: str = "unit.physical.mass.mass"
-
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", "method", allow_reuse=True, pre=True
-    )(ensure_list)
+    path_unit: str = "unit.physical.mass.value"
 
 
-class Phase(Property):
+class Phase(Parameter):
     H: Value = Value(**{})
     N: Optional[float] = np.nan
     G1: Value = Value(**{})
@@ -269,15 +257,8 @@ class Phase(Property):
     facility: Optional[str] = ""
     name_filter: Optional[str] = ""
 
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", allow_reuse=True, pre=True
-    )(ensure_list)
 
-    def __str__(self):
-        return self.json()
-
-
-class PhaseFunction(Property):
+class PhaseFunction(Parameter):
     # Generic
     generic_johnson_v: Phase = pydantic.Field(Phase(**{}), alias="Generic/Johnson.V")
     # ATLAS
@@ -285,78 +266,69 @@ class PhaseFunction(Property):
     misc_atlas_orange: Phase = pydantic.Field(Phase(**{}), alias="Misc/Atlas.orange")
 
 
-class Spin(Property):
-    period: Value = Value(**{})
-    t0: Optional[float] = np.nan
-    Wp: Optional[float] = np.nan
-    lat: Value = Value(**{})
-    RA0: Optional[float] = np.nan
-    DEC0: Optional[float] = np.nan
-    long_: Value(**{}) = pydantic.Field(Value(**{}), alias="long")
-    method: Optional[List[Method]] = [Method(**{})]
-    bibref: Optional[List[Bibref]] = [Bibref(**{})]
+class Spin(Parameter):
+    period: List[Value] = [Value(**{})]
+    t0: List[float] = [np.nan]
+    Wp: List[float] = [np.nan]
+    lat: List[Value] = [Value(**{})]
+    RA0: List[float] = [np.nan]
+    DEC0: List[float] = [np.nan]
+    long_: List[Value] = pydantic.Field([Value(**{})], alias="long")
+    method: List[List[Method]] = [[Method(**{})]]
+    bibref: List[List[Bibref]] = [[Bibref(**{})]]
 
-    def __str__(self):
-        return self.json()
-
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", "method", allow_reuse=True, pre=True
-    )(ensure_list)
+    path_unit: str = "unit.physical.spin.value"
 
 
-class Taxonomy(Property):
-    class_: Optional[str] = pydantic.Field("", alias="class")
-    scheme: Optional[str] = ""
-    bibref: List[Bibref] = [Bibref(**{})]
-    method: List[Method] = [Method(**{})]
-    waverange: Optional[str] = ""
-
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", "method", allow_reuse=True, pre=True
-    )(ensure_list)
+class Taxonomy(Parameter):
+    class_: List[str] = pydantic.Field([""], alias="class")
+    scheme: List[str] = [""]
+    bibref: List[List[Bibref]] = [[Bibref(**{})]]
+    method: List[List[Method]] = [[Method(**{})]]
+    waverange: List[str] = [""]
 
     def __str__(self):
         if not self.class_:
             return "No taxonomy on record."
-        return self.class_
+        return ", ".join(self.class_)
 
 
-class ThermalInertia(Property):
+class ThermalInertia(Parameter):
     TI: Value = Value(**{})
     dsun: Optional[float] = np.nan
     bibref: List[Bibref] = []
     method: List[Method] = []
 
-    _ensure_list: classmethod = pydantic.validator(
-        "bibref", "method", allow_reuse=True, pre=True
-    )(ensure_list)
+
+class AbsoluteMagnitude(Value):
+    G: Optional[float] = np.nan
+    bibref: List[Bibref] = []
 
 
-class PhysicalParameters(Property):
+class PhysicalParameters(Parameter):
     mass: Mass = Mass(**{})
-    spin: List[Spin] = [Spin(**{})]
+    spin: Spin = Spin(**{})
     colors: Colors = Colors(**{})
     albedo: Albedo = Albedo(**{})
+    density: Density = Density(**{})
     diameter: Diameter = Diameter(**{})
     taxonomy: Taxonomy = Taxonomy(**{})
     phase_function: PhaseFunction = PhaseFunction(**{})
     thermal_inertia: ThermalInertia = ThermalInertia(**{})
+    absolute_magnitude: AbsoluteMagnitude = AbsoluteMagnitude(**{})
 
-    def __str__(self):
-        return self.json()
-
-    _ensure_list: classmethod = pydantic.validator("spin", allow_reuse=True, pre=True)(
-        ensure_list
-    )
-
-    _ensure_unique_taxonomy: classmethod = pydantic.validator(
+    _ensure_list: classmethod = pydantic.validator(
         "taxonomy", allow_reuse=True, pre=True
-    )(ensure_unique_taxonomy)
+    )(ensure_list)
+
+    _merge_entries: classmethod = pydantic.validator(
+        "spin", "taxonomy", allow_reuse=True, pre=True
+    )(merge_entries)
 
 
 # ------
 # Equation of state
-class EqStateVector(Property):
+class EqStateVector(Parameter):
     ref_epoch: Optional[float] = np.nan
     position: List[float] = [np.nan, np.nan, np.nan]
     velocity: List[float] = [np.nan, np.nan, np.nan]
@@ -364,38 +336,32 @@ class EqStateVector(Property):
 
 # ------
 # Highest level branches
-class Parameters(Property):
+class Parameters(Parameter):
     physical: PhysicalParameters = PhysicalParameters(**{})
     dynamical: DynamicalParameters = DynamicalParameters(**{})
     eq_state_vector: EqStateVector = EqStateVector(**{})
 
-    def __str__(self):
-        return self.json()
-
     class Config:
         arbitrary_types_allowed = True
 
 
-class Link(Property):
+class Link(Parameter):
     unit: Optional[str] = ""
     self_: Optional[str] = pydantic.Field("", alias="self")
     quaero: Optional[str] = ""
     description: Optional[str] = ""
 
 
-class Ssocard(Property):
+class Ssocard(Parameter):
     version: Optional[str] = ""
     datetime: Optional[dt.datetime] = None
 
 
-class Datacloud(Property):
+class Datacloud(Parameter):
     """The collection of links to datacloud catalogue associated to this ssoCard."""
 
-    aams: Optional[str] = ""
-    astdys: Optional[str] = ""
     astorb: Optional[str] = ""
-    binarymp_tab: Optional[str] = ""
-    binarymp_ref: Optional[str] = ""
+    binarymp: Optional[str] = ""
     diamalbedo: Optional[str] = ""
     families: Optional[str] = ""
     masses: Optional[str] = ""
@@ -413,7 +379,7 @@ class Rock(pydantic.BaseModel):
     name: Optional[str] = ""
     type_: Optional[str] = pydantic.Field("", alias="type")
     class_: Optional[str] = pydantic.Field("", alias="class")
-    number: Optional[int] = np.nan
+    number: Optional[int] = None
     parent: Optional[str] = ""
     system: Optional[str] = ""
 
@@ -426,17 +392,23 @@ class Rock(pydantic.BaseModel):
     datacloud: Datacloud = Datacloud(**{})
 
     # the catalogues
-    taxonomies: rocks.datacloud.Taxonomies = rocks.datacloud.Taxonomies(**{})
-    aams: rocks.datacloud.AAMS = rocks.datacloud.AAMS(**{})
-    astdys: rocks.datacloud.AstDyS = rocks.datacloud.AstDyS(**{})
     astorb: rocks.datacloud.Astorb = rocks.datacloud.Astorb(**{})
+    binarymp: rocks.datacloud.Binarymp = rocks.datacloud.Binarymp(**{})
+    colors: rocks.datacloud.Colors = rocks.datacloud.Colors(**{})
     diamalbedo: rocks.datacloud.Diamalbedo = rocks.datacloud.Diamalbedo(**{})
+    families: rocks.datacloud.Families = rocks.datacloud.Families(**{})
     masses: rocks.datacloud.Masses = rocks.datacloud.Masses(**{})
     mpcatobs: rocks.datacloud.Mpcatobs = rocks.datacloud.Mpcatobs(**{})
+    mpcorb: rocks.datacloud.Mpcorb = rocks.datacloud.Mpcorb(**{})
     pairs: rocks.datacloud.Pairs = rocks.datacloud.Pairs(**{})
+    phase_functions: rocks.datacloud.PhaseFunction = rocks.datacloud.PhaseFunction(**{})
     taxonomies: rocks.datacloud.Taxonomies = rocks.datacloud.Taxonomies(**{})
+    thermal_properties: rocks.datacloud.ThermalProperties = (
+        rocks.datacloud.ThermalProperties(**{})
+    )
+    yarkovskies: rocks.datacloud.Yarkovskies = rocks.datacloud.Yarkovskies(**{})
 
-    def __init__(self, id_, ssocard={}, datacloud=[], skip_id_check=False):
+    def __init__(self, id_, ssocard=None, datacloud=None, skip_id_check=False):
         """Identify a minor body  and retrieve its properties from SsODNet.
 
         Parameters
@@ -471,7 +443,7 @@ class Rock(pydantic.BaseModel):
         'C'
         >>> ceres.taxonomy.shortbib
         'DeMeo+2009'
-        >>> ceres.diameter
+        >>> ceres.diameter.value
         848.4
         >>> ceres.diameter.unit
         'km'
@@ -486,17 +458,28 @@ class Rock(pydantic.BaseModel):
 
         # Get ssoCard and datcloud catalogues
         if not pd.isnull(id_):
-            if not ssocard:
+            if ssocard is None:
                 ssocard = rocks.ssodnet.get_ssocard(id_)
 
             if ssocard is None:
-                # Asteroid does not have an ssoCard. Instantiate minimal ssoCard for meaningful error output.
+                # Asteroid does not have an ssoCard
+                # Instantiate minimal ssoCard for meaningful error output.
                 ssocard = {"name": id_provided}
-                print(f"Did not find ssoCard for asteroid '{id_provided}'.")
+
+                rich.print(
+                    f"Error 404: missing ssoCard for [green]{id_provided}[/green]."
+                )
+                # This only gets printed once
+                warnings.warn(
+                    "See https://rocks.readthedocs.io/en/latest/tutorials.html#error-404 for help."
+                )
 
             else:
-                for catalogue in datacloud:
-                    ssocard = self.__add_datacloud_catalogue(id_, catalogue, ssocard)
+                if datacloud is not None:
+                    for catalogue in datacloud:
+                        ssocard = self.__add_datacloud_catalogue(
+                            id_, catalogue, ssocard
+                        )
         else:
             # Something failed. Instantiate minimal ssoCard for meaningful error output.
             ssocard = {"name": id_provided}
@@ -505,12 +488,39 @@ class Rock(pydantic.BaseModel):
         try:
             super().__init__(**ssocard)  # type: ignore
         except pydantic.ValidationError as message:
+
             self.__parse_error_message(message, id_, ssocard)
-            return super().__init__(**{"name": id_provided})
+
+            # Set the offending properties to None to allow for instantiation anyway
+            for error in message.errors():
+
+                # Dynamically remove offending parts of the ssoCard
+                offending_part = ssocard
+
+                for location in error["loc"][:-1]:
+                    offending_part = offending_part[location]
+
+                del offending_part[error["loc"][-1]]
+
+            super().__init__(**ssocard)  # type: ignore
+
+        # Convert the retrieve datacloud catalogues into DataCloudDataFrame objects
+        if datacloud is not None:
+            for catalogue in datacloud:
+
+                if catalogue in ["diameters", "albedos"]:
+                    catalogue = "diamalbedo"
+
+                setattr(
+                    self,
+                    catalogue,
+                    rocks.datacloud.DataCloudDataFrame(
+                        data=getattr(self, catalogue).dict()
+                    ),
+                )
 
     def __getattr__(self, name):
-        """Implement attribute shortcuts: omission of parameters.physical/dynamical.
-        Gets called if getattribute fails."""
+        """Implement attribute shortcuts. Gets called if __getattribute__ fails."""
 
         # These are shortcuts
         if name in self.__aliases["physical"].values():
@@ -519,6 +529,7 @@ class Rock(pydantic.BaseModel):
         if name in self.__aliases["dynamical"].values():
             return getattr(self.parameters.dynamical, name)
 
+        # TODO This could be coded in a more abstract way
         # These are proper aliases
         if name in self.__aliases["orbital_elements"].keys():
             return getattr(
@@ -532,12 +543,18 @@ class Rock(pydantic.BaseModel):
                 self.__aliases["proper_elements"][name],
             )
 
+        if name in self.__aliases["physical"].keys():
+            return getattr(
+                self.parameters.physical,
+                self.__aliases["physical"][name],
+            )
+
         if name in self.__aliases["diamalbedo"]:
             return getattr(self, "diamalbedo")
 
         raise AttributeError(
             f"'Rock' object has no attribute '{name}'. Run "
-            f"'rocks properties' to get a list of accepted properties."
+            f"'rocks parameters' to get a list of accepted properties."
         )
 
     def __repr__(self):
@@ -553,7 +570,8 @@ class Rock(pydantic.BaseModel):
         return hash(self.id_)
 
     def __add_datacloud_catalogue(self, id_, catalogue, data):
-        """Retrieve datacloud catalogue for asteroid and deserialize."""
+        """Retrieve datacloud catalogue for asteroid and deserialize into
+        pydantic model."""
 
         if catalogue not in rocks.datacloud.CATALOGUES.keys():
             raise ValueError(
@@ -568,42 +586,46 @@ class Rock(pydantic.BaseModel):
         # retrieve the catalogue
         cat = rocks.ssodnet.get_datacloud_catalogue(id_, catalogue_ssodnet)
 
-        if cat is None:
+        if cat is None or not cat:
             return data
 
         # turn list of dict (catalogue entries) into dict of list
         cat = {
             key: [c[key] for c in cat]
-            if catalogue not in ["aams", "astdys", "astorb", "pairs", "families"]
+            if catalogue not in ["aams", "astorb", "pairs", "families"]
             else cat[0][key]
             for key in cat[0].keys()
         }
 
         # add 'preferred' attribute where applicable
-        if catalogue in ["taxonomies", "masses"]:
+        if catalogue_ssodnet in ["taxonomy", "masses", "diamalbedo"]:
             cat["preferred"] = [False] * len(list(cat.values())[0])
-        elif catalogue_ssodnet in ["diamalbedo"]:
+        if catalogue_ssodnet in ["diamalbedo"]:
             cat["preferred_albedo"] = [False] * len(list(cat.values())[0])
             cat["preferred_diameter"] = [False] * len(list(cat.values())[0])
 
         # add catalogue to Rock
         data[catalogue_attribute] = cat
-
         return data
 
     def __parse_error_message(self, message, id_, data):
         """Print informative error message if ssocard data is invalid."""
-        print(f"{id_}:")
+        print(f"\n{id_}:")
 
         # Look up offending value in ssoCard
         for error in message.errors():
             value = data
+
             for loc in error["loc"]:
-                value = value[loc]
+                try:
+                    value = value[loc]
+                except TypeError:
+                    break
 
             rich.print(
                 f"Error: {' -> '.join([str(e) for e in error['loc']])} is invalid: {error['msg']}\n"
                 f"Passed value: {value}\n"
+                f"Replacing value with empty default to continue.\n"
             )
 
     __aliases = {
@@ -615,14 +637,17 @@ class Rock(pydantic.BaseModel):
             "parameters.dynamical.pair": "pair",
         },
         "physical": {
-            "parameters.physical.diameter": "diameter",
+            "H": "absolute_magnitude",
+            "parameters.physical.absolute_magnitude": "absolute_magnitude",
             "parameters.physical.albedo": "albedo",
             "parameters.physical.colors": "colors",
+            "parameters.physical.diameter": "diameter",
+            "parameters.physical.density": "density",
             "parameters.physical.mass": "mass",
-            "parameters.physical.thermal_properties": "thermal_properties",
+            "parameters.physical.phase_function": "phase_function",
             "parameters.physical.spin": "spin",
             "parameters.physical.taxonomy": "taxonomy",
-            "parameters.physical.phase": "phase",
+            "parameters.physical.thermal_properties": "thermal_properties",
         },
         "orbital_elements": {
             "a": "semi_major_axis",
@@ -638,12 +663,12 @@ class Rock(pydantic.BaseModel):
     }
 
 
-def rocks_(identifier, datacloud=[], progress=False):
+def rocks_(ids, datacloud=None, progress=False):
     """Create multiple Rock instances.
 
     Parameters
     ==========
-    identifier : list of str, list of int, list of float, np.array, pd.Series
+    ids : list of str, list of int, list of float, np.array, pd.Series
         An iterable containing minor body identifiers.
     datacloud : list of str
         List of additional catalogues to retrieve from datacloud.
@@ -656,25 +681,40 @@ def rocks_(identifier, datacloud=[], progress=False):
     list of rocks.core.Rock
         A list of Rock instances
     """
-    if isinstance(identifier, pd.Series):
-        identifier = identifier.values
 
-    if len(identifier) == 1:
-        ids = [rocks.identify(identifier, return_id=True, progress=progress)[-1]]
+    # Get IDs
+    if len(ids) == 1 or isinstance(ids, str):
+        ids = [rocks.identify(ids, return_id=True, progress=progress)[-1]]
 
     else:
-        ids = [
-            id_
-            for _, _, id_ in rocks.identify(
-                identifier, return_id=True, progress=progress
+        _, _, ids = zip(*rocks.identify(ids, return_id=True, progress=progress))
+
+    # Load ssoCards asynchronously
+    rocks.ssodnet.get_ssocard(
+        [id_ for id_ in ids if not id_ is None], progress=progress
+    )
+
+    if datacloud is not None:
+
+        if isinstance(datacloud, str):
+            datacloud = [datacloud]
+
+        # Load datacloud catalogues asynchronously
+        for cat in datacloud:
+
+            if cat not in rocks.datacloud.CATALOGUES.keys():
+                raise ValueError(
+                    f"Unknown datacloud catalogue name: '{cat}'"
+                    f"\nChoose from {rocks.datacloud.CATALOGUES.keys()}"
+                )
+
+            rocks.ssodnet.get_datacloud_catalogue(
+                [id_ for id_ in ids if not id_ is None], cat, progress=progress
             )
-        ]
 
     rocks_ = [
-        Rock(id_, skip_id_check=True, datacloud=datacloud)
-        for id_ in tqdm(
-            ids, desc="Building rocks : ", total=len(ids), disable=not progress
-        )
+        Rock(id_, skip_id_check=True, datacloud=datacloud) if not id_ is None else None
+        for id_ in ids
     ]
 
     return rocks_
