@@ -1,7 +1,6 @@
 #!/usr/bin/env python
 
-"""Implement asteroid identification module.
-"""
+"""Local and remote asteroid name resolution."""
 import asyncio
 import re
 import warnings
@@ -10,7 +9,7 @@ import aiohttp
 import nest_asyncio
 import numpy as np
 import pandas as pd
-from tqdm import tqdm
+from rich.progress import Progress
 
 import rocks
 
@@ -18,90 +17,100 @@ import rocks
 nest_asyncio.apply()
 
 
-def identify(id_, return_id=False, progress=False):
+def identify(id_, return_id=False, local=True, progress=False):
     """Resolve names and numbers of one or more minor bodies using identifiers.
 
     Parameters
-    ==========
-    id_ : str, int, float, list, set, np.ndarray, pd.Series
+    ----------
+    id_ : str, int, float, list, range, set, np.ndarray, pd.Series
         One or more identifying names or numbers to resolve.
     return_id : bool
         Return the SsODNet ID of the asteroid as third member of
         the tuple. Default is False.
-    progress : bool or tdqm.std.tqdm
-        If progress is True, this is a progress bar instance. Else, it's False.
+    local : bool
+        Try resolving the name locally first. Default is True.
+    progress : bool
+        Show progress bar. Default is False.
 
     Returns
-    =======
+    -------
     tuple, list of tuple : (str, int, str), (None, np.nan, None)
         List containing len(id_) tuples. Each tuple contains the asteroid's
-        name, number, and SsODNet ID if the identifier was resolved. Otherwise,
-        the values are None for name and SsODNet and np.nan for the number.
-        If a single asteroid is identified, a tuple is returned.
+        name, number, and the SsODNet ID if return_id=True. If the resolution
+        failed, the values are None for name and SsODNet and np.nan for the
+        number. If a single identifier is resolved, a tuple is returned.
 
     Notes
-    =====
+    -----
     Name resolution is first attempted locally, then remotely via quaero. If
-    the asteroid is unnumbered, it's number is returned as np.nan.
+    the asteroid is unnumbered, its number is np.nan.
     """
+
+    # ------
+    # Verify input
     if isinstance(id_, (str, int, float)):
         id_ = [id_]
     elif isinstance(id_, pd.Series):
-        id_ = id_.values
+        id_ = id_.to_list()
+    elif isinstance(id_, np.ndarray):
+        id_ = id_.tolist()
     elif isinstance(id_, (set, range)):
         id_ = list(id_)
     elif id_ is None:
         warnings.warn(f"Received id_ of type {type(id_)}.")
-        return (None, np.nan)
+        return (None, np.nan) if not return_id else (None, np.nan, None)  # type: ignore
     elif not isinstance(id_, (list, np.ndarray)):
         raise TypeError(
             f"Received id_ of type {type(id_)}, expected one of: "
-            f"str, int, float, list, np.ndarray, pd.Series"
+            f"str, int, float, list, set, range, np.ndarray, pd.Series"
         )
 
-    if progress:
-        progress = tqdm(desc="Identifying rocks : ", total=len(id_))
+    if not id_:
+        warnings.warn("Received empty list of identifiers.")
+        return (None, np.nan) if not return_id else (None, np.nan, None)  # type: ignore
 
-    # Run async loop to resolve names
-    loop = asyncio.get_event_loop()
-    results = loop.run_until_complete(_identify(id_, progress))
+    # ------
+    # Run asynchronous event loop for name resolution
+    with Progress(disable=not progress) as progress_bar:
 
-    if progress:
-        progress.close()
+        task = progress_bar.add_task("Identifying rocks", total=len(id_))  # type: ignore
+        loop = asyncio.get_event_loop()
+        results = loop.run_until_complete(_identify(id_, local, progress_bar, task))
 
+        # ------
+        # Check if any failed due to 502 and rerun them
+        idx_failed = [
+            i for i, result in enumerate(results) if result == (None, None, None)
+        ]
+
+        if idx_failed:
+            results = np.array(results)
+            results[idx_failed] = loop.run_until_complete(
+                _identify(np.array(id_)[idx_failed], local, progress_bar, task)
+            )
+            results = results.tolist()
+
+    # ------
+    # Verify the output format
     if not return_id:
         results = [r[:2] for r in results]
 
-    if len(id_) == 1:
+    if len(id_) == 1:  # type: ignore
         results = results[0]
 
-    return results
+    return results  # type: ignore
 
 
-async def _identify(id_, progress):
-    """Resolve asteroid name asynchronously. First attempts local lookup, then
-    queries quaero.
-
-    Parameters
-    ==========
-    id_ : str, int, float, list, set, np.ndarray, pd.Series
-        One or more identifying names or numbers to resolve.
-
-    Returns
-    =======
-    tuple : (str, int or float, str), (None, np.nan, None)
-        Tuple containing the asteroid's name, number, and SsODNet ID if the
-        identifier was resolved. Otherwise, the values are None for name and
-        SsODNet and np.nan for the number.
-    progress : bool or tdqm.std.tqdm
-       If progress is True, this is a progress bar instance. Else, it's False.
-    """
-    INDEX = rocks.utils.read_index()
+async def _identify(id_, local, progress_bar, task):
+    """Establish the asynchronous HTTP session and launch the name resolution."""
+    INDEX = rocks.utils.load_index()
 
     async with aiohttp.ClientSession(timeout=aiohttp.ClientTimeout()) as session:
 
         tasks = [
-            asyncio.ensure_future(_query_and_resolve(i, session, INDEX, progress))
+            asyncio.ensure_future(
+                _resolve(i, session, local, progress_bar, task, INDEX)
+            )
             for i in id_
         ]
 
@@ -110,63 +119,75 @@ async def _identify(id_, progress):
     return results
 
 
-async def _query_and_resolve(id_, session, INDEX, progress):
-    """Standardize identifier, do local look-up, else query quaero and parser
-    methods asynchronously. Call with identify function."""
-
-    if progress:
-        progress.update()
-
-    id_ = standardize_id_(id_)
-
-    # Try local resolution
-    if isinstance(id_, (int)):
-        if id_ in INDEX.number.values:
-            name, ssodnet_id = INDEX.loc[INDEX.number == id_, ["name", "id_"]].iloc[0]
-            return (name, id_, ssodnet_id)
-    elif isinstance(id_, (str)):
-        if id_ in INDEX.name.values:
-            number, ssodnet_id = INDEX.loc[INDEX.name == id_, ["number", "id_"]].iloc[0]
-            return (id_, number, ssodnet_id)
-        elif id_ in INDEX.id_.values:
-            name, number = INDEX.loc[INDEX.id_ == id_, ["name", "number"]].iloc[0]
-            return (name, number, id_)
+async def _resolve(id_, session, local, progress_bar, task, INDEX):
+    """Resolve the identifier locally or remotely."""
 
     if pd.isnull(id_) or not id_:  # covers None, np.nan, empty string
+        warnings.warn("Received empty or NaN identifier.")
+        progress_bar.update(task, advance=1)
         return (None, np.nan, None)
 
-    # Local resolution failed, do remote query
-    response = await _query_quaero(id_, session)
+    if local:
+        success, (name, number, ssodnet_id) = _local_lookup(id_, INDEX)
 
-    if response:
-        name, number, ssodnet_id = _parse_quaero_response(response["data"], str(id_))
-        if isinstance(ssodnet_id, str):
+        if success:
+            progress_bar.update(task, advance=1)
             return (name, number, ssodnet_id)
 
-    return (None, np.nan, None)
+    # Local resolution failed, do remote query
+    id_ = _standardize_id(id_)
+    response = await _query_quaero(id_, session)
+
+    if response is None:  # query failed with 502
+        return (None, None, None)
+
+    if not response:  # remote resolution failed
+        progress_bar.update(task, advance=1)
+        return (None, np.nan, None)
+
+    progress_bar.update(task, advance=1)
+    return _parse_quaero_response(response["data"], str(id_))
 
 
-def standardize_id_(id_):
+def _local_lookup(id_, INDEX):
+    """Perform local index resolution."""
+
+    # Check for int, float, numeric str
+    if isinstance(id_, (int, float)) or (isinstance(id_, str) and id_.isnumeric()):
+        id_ = int(id_)
+
+        if id_ in INDEX["number"]:
+            name, ssodnet_id = INDEX["number"][id_]
+            return True, (name, id_, ssodnet_id)
+        else:
+            return False, (None, np.nan, None)
+
+    # It's a name or SsODNet ID - reduce the identifier
+    reduced = rocks.utils.reduce_id(id_)
+
+    if reduced in INDEX["reduced"]:
+        name, number, id_ = INDEX["reduced"][reduced]
+        return True, (name, number, id_)
+    else:
+        return False, (None, np.nan, None)
+
+
+def _standardize_id(id_):
     """Try to infer id_ type and re-format if necessary to ensure
-    successful lookup.
+    successful remote lookup.
 
     Parameters
-    ==========
+    ----------
     id_ : str, int, float
         The minor body's name, designation, or number.
 
     Returns
-    =======
+    -------
     str, int, float, None
         The standardized name, designation, or number. None if id_ is NaN or None.
     """
-    if isinstance(id_, (int, float, np.int64)):
-        try:
-            id_ = int(id_)
-        except ValueError:  # np.nan
-            if np.isnan(id_):
-                warnings.warn(f"Received id 'NaN'.")
-                return None
+    if isinstance(id_, (int, float)):
+        return int(id_)
 
     elif isinstance(id_, str):
         # String id_. Perform some regex tests to make sure it's well formatted
@@ -178,14 +199,10 @@ def standardize_id_(id_):
         except ValueError:
             pass
 
-        # Empty string
-        if not id_:
-            return None
-
         # Asteroid name
-        elif re.match(r"^[A-Za-z ]*$", id_):
+        if re.match(r"^[A-Za-z ]*$", id_):
             # guess correct capitalization
-            id_ = " ".join([sub.capitalize() for sub in id_.split(" ")])  # type: ignore
+            id_ = " ".join([sub.capitalize() for sub in id_.split(" ")])
 
         # Asteroid designation
         elif re.match(
@@ -196,7 +213,7 @@ def standardize_id_(id_):
 
             # Ensure whitespace between year and id_
             id_ = re.sub(r"[\W_]+", "", id_)
-            ind = re.search(r"[A18920]{1,2}[0-9]{2}", id_).end()
+            ind = re.search(r"[A18920]{1,2}[0-9]{2}", id_).end()  # type: ignore
             id_ = f"{id_[:ind]} {id_[ind:]}"
 
             # Replace A by 1
@@ -217,39 +234,34 @@ def standardize_id_(id_):
             pass
 
         # Remaining should be unconventional asteroid names like
-        # "G!kun||'homdima" or packed designaitons
+        # "G!kun||'homdima" or packed designations
         else:
             pass
     else:
-        if id_ is None:
-            warnings.warn(f"Received id 'None'.")
-            return None
-        else:
-            warnings.warn(
-                f"Did not understand type of id: {type(id_)}"
-                f"\nShould be integer, float, or string."
-            )
+        warnings.warn(
+            f"Did not understand type of id: {type(id_)}"
+            f"\nShould be integer, float, or string."
+        )
     return id_
 
 
-# ------
-# Quaero query methods
 async def _query_quaero(id_, session):
     """Query quaero and parse result for a single object.
 
     Parameters
-    ==========
+    ----------
     id_ : str, int, float
         Asteroid name, number, or designation.
     session : aiohttp.ClientSession
         asyncio session
 
     Returns
-    =======
-    dict or False
-        Quaero response in json format if successful. False if query failed.
+    -------
+    dict, None, or False
+        Quaero response in json format if successful. None if ContentTypeError.
+        False if query failed.
     """
-    # Build query
+
     url = "https://api.ssodnet.imcce.fr/quaero/1/sso/search"
 
     params = {
@@ -259,44 +271,48 @@ async def _query_quaero(id_, session):
     }
 
     response = await session.request(method="GET", url=url, params=params)
-    response_json = await response.json()
 
-    # No match found
-    if "data" not in response_json.keys():
+    try:
+        response = await response.json(content_type=None)
+    except aiohttp.ContentTypeError:
+        return None
+
+    if "data" not in response.keys():  # no match found
         warnings.warn(f"Could not find data for id {id_}.")
         return False
-    # Data is empty
-    elif not response_json["data"]:
+
+    elif not response["data"]:  # empty response
         warnings.warn(f"Could not find match for id {id_}.")
         return False
-    else:
-        return response_json
+
+    return response
 
 
-def _parse_quaero_response(data_json, id_):
+def _parse_quaero_response(data, id_):
     """Parse JSON response from Quaero.
 
     Parameters
-    ==========
-    data_json : list of dict
+    ----------
+    data : list of dict
         Quaero query response in json format.
     id_ : str, int, float
         Asteroid name, number, or designation.
 
     Returns
-    =======
+    -------
     tuple : (str, int, str), (None, np.nan, None)
         Tuple containing the asteroid's name, number, and SsODNet ID if the
         identifier was resolved. Otherwise, the values are None for name and
         SsODNet and np.nan for the number.
     """
 
-    # convert all ids to lowercase strings for capitals-agnostic comparison
     id_ = str(id_).lower()
 
-    for match in data_json:
+    for match in data:
         if match["name"].lower() == id_:
             break
+        elif match["id"].lower() == id_:
+            break
         elif any([alias.lower() == id_ for alias in match["aliases"]]):
             break
     else:
