@@ -11,39 +11,39 @@ import aiohttp
 import json
 import numpy as np
 import pandas as pd
-from tqdm import tqdm
+from rich.progress import Progress
 
 import rocks
 
 
-def get_ssocard(id_ssodnet, progress=False, no_cache=False):
+def get_ssocard(id_ssodnet, progress=False, local=True):
     """Retrieve the ssoCard of one or many asteroids, using their SsODNet IDs.
 
     Parameters
-    ==========
+    ----------
     id_ssodnet : str, list, np.ndarray, pd.series
         one or more ssodnet ids.
     progress : bool
         Show progressbar. Default is False.
-    no_cache : bool
-        If True, forces the remote query of the ssoCard. Default is False.
+    local : bool
+        If False, forces the remote query of the ssoCard. Default is True.
 
     Returns
-    =======
+    -------
     dict, list of dict
         list containing len(id_) dictionaries corresponding to the ssocards of
         the passed identifiers. if the card is not available, the dict is empty.
         If a single card is retrieved, a dict is returned.
 
     Notes
-    =====
-    card retrieval is first attempted locally, then remotely via datacloud.
+    -----
+    Card retrieval is first attempted locally, then remotely via datacloud.
     """
     if isinstance(id_ssodnet, str):
         id_ssodnet = [id_ssodnet]
     elif isinstance(id_ssodnet, pd.Series):
         id_ssodnet = id_ssodnet.values
-    elif isinstance(id_ssodnet, set):
+    elif isinstance(id_ssodnet, (set, tuple)):
         id_ssodnet = list(id_ssodnet)
     elif id_ssodnet is None:
         warnings.warn(f"Received SsODNet ID of type {type(id_ssodnet)}.")
@@ -54,13 +54,15 @@ def get_ssocard(id_ssodnet, progress=False, no_cache=False):
             f"str, list, np.ndarray, pd.Series"
         )
 
-    if progress:
-        progress = tqdm(desc="Getting ssoCards : ", total=len(id_ssodnet))
-
     # ---
     # Run async loop to get ssoCard
-    loop = asyncio.get_event_loop()
-    cards = loop.run_until_complete(_get_ssocard(id_ssodnet, progress, no_cache))
+    with Progress(disable=not progress) as progress_bar:
+
+        progress = progress_bar.add_task("Getting ssoCards", total=len(id_ssodnet))
+        loop = asyncio.get_event_loop()
+        cards = loop.run_until_complete(
+            _get_ssocard(id_ssodnet, progress_bar, progress, local)
+        )
 
     if len(id_ssodnet) == 1:
         cards = cards[0]
@@ -68,29 +70,14 @@ def get_ssocard(id_ssodnet, progress=False, no_cache=False):
     return cards
 
 
-async def _get_ssocard(id_ssodnet, progress, no_cache):
-    """Get ssoCard asynchronously. First attempts local lookup, then
-    queries SsODNet.
-
-    Parameters
-    ==========
-    id_ssodnet : str, list, np.ndarray, pd.series
-        one or more ssodnet ids.
-    progress : bool or tdqm.std.tqdm
-       If progress is True, this is a progress bar instance. Else, it's False.
-    no_cache : bool
-        If True, forces the remote query of the ssoCard.
-
-    Returns
-    =======
-    list of dict
-        list containing len(id_) dictionaries corresponding to the ssocards of
-    the passed identifiers. if the card is not available, the dict is empty.
-    """
+async def _get_ssocard(id_ssodnet, progress_bar, progress, local):
+    """Get ssoCard asynchronously. First attempt local lookup, then query SsODNet."""
     async with aiohttp.ClientSession(timeout=aiohttp.ClientTimeout()) as session:
 
         tasks = [
-            asyncio.ensure_future(_local_or_remote(i, session, progress, no_cache))
+            asyncio.ensure_future(
+                _local_or_remote(i, session, progress_bar, progress, local)
+            )
             for i in id_ssodnet
         ]
 
@@ -99,26 +86,25 @@ async def _get_ssocard(id_ssodnet, progress, no_cache):
     return results
 
 
-async def _local_or_remote(id_ssodnet, session, progress, no_cache):
+async def _local_or_remote(id_ssodnet, session, progress_bar, progress, local):
     """Check for presence of ssoCard in cache directory. Else, query from SsODNet."""
 
-    if progress:
-        progress.update()
-
     PATH_CARD = os.path.join(rocks.PATH_CACHE, f"{id_ssodnet}.json")
 
-    if os.path.isfile(PATH_CARD) and not no_cache:
+    if os.path.isfile(PATH_CARD) and local:
         with open(PATH_CARD, "r") as file_card:
+            progress_bar.update(progress, advance=1)
             return json.load(file_card)[id_ssodnet]
 
     # Local retrieval failed, do remote query
     card = await _query_ssodnet(id_ssodnet, session)
 
     # save to cache
-    if card:
+    if card and card is not None:
         with open(PATH_CARD, "w") as file_card:
             json.dump(card, file_card)
 
+    progress_bar.update(progress, advance=1)
     return card[id_ssodnet]
 
 
@@ -126,14 +112,14 @@ async def _query_ssodnet(id_ssodnet, session):
     """Query quaero and parse result for a single object.
 
     Parameters
-    ==========
+    ----------
     id_ssodnet : str
         Asteroid ID from SsODNet.
     session : aiohttp.ClientSession
         asyncio session
 
     Returns
-    =======
+    -------
     dict
         SsODNet response as dict if successful. Empty if query failed.
     """
@@ -151,20 +137,22 @@ async def _query_ssodnet(id_ssodnet, session):
 
 
 # ------
-def get_datacloud_catalogue(id_ssodnet, catalogue, progress=False):
+def get_datacloud_catalogue(id_ssodnet, catalogue, progress=False, local=True):
     """Retrieve the datacloud catalogue of one or many asteroids, using their SsODNet IDs.
 
     Parameters
-    ==========
+    ----------
     id_ssodnet : str, list, np.ndarray, pd.series
-        The ssodnet id of the asteroid
-    catalogue : str
-        The name of the datacloud catalogue to retrieve.
+        The ssodnet id of the asteroid. Can be one or many.
+    catalogue : str, list of str
+        The name of the datacloud catalogue to retrieve. Can be one or many.
     progress : bool
         Show progressbar. Default is False.
+    local : bool
+        If False, forces the remote query of the ssoCard. Default is True.
 
     Returns
-    =======
+    -------
     list of dict, list of list of dict
         list containing len(catalogue) dictionaries corresponding to
         the catalogues of the passed identifier. If the catalogue is
@@ -173,14 +161,14 @@ def get_datacloud_catalogue(id_ssodnet, catalogue, progress=False):
        If progress is True, this is a progress bar instance. Else, it's False.
 
     Notes
-    =====
+    -----
     Catalogue retrieval is first attempted locally, then remotely via datacloud.
     """
     if isinstance(id_ssodnet, str):
         id_ssodnet = [id_ssodnet]
     elif isinstance(id_ssodnet, pd.Series):
         id_ssodnet = id_ssodnet.values
-    elif isinstance(id_ssodnet, set):
+    elif isinstance(id_ssodnet, (set, tuple)):
         id_ssodnet = list(id_ssodnet)
     elif id_ssodnet is None:
         warnings.warn(f"Received SsODNet ID of type {type(id_ssodnet)}.")
@@ -202,31 +190,36 @@ def get_datacloud_catalogue(id_ssodnet, catalogue, progress=False):
     # Flatten input for easier calling
     id_catalogue = list(product(id_ssodnet, catalogue))
 
-    if progress:
-        progress = tqdm(desc="Getting catalogues : ", total=len(id_catalogue))
+    with Progress(disable=not progress) as progress_bar:
 
-    # Run async loop to get ssoCard
-    loop = asyncio.get_event_loop()
-    catalogues = loop.run_until_complete(
-        _get_datacloud_catalogue(id_catalogue, progress)
-    )[0]
+        progress = progress_bar.add_task(
+            "Getting catalogues" if len(catalogue) > 1 else catalogue[0],
+            total=len(id_catalogue),
+        )
+
+        # Run async loop to get ssoCard
+        loop = asyncio.get_event_loop()
+        catalogues = loop.run_until_complete(
+            _get_datacloud_catalogue(id_catalogue, progress_bar, progress, local)
+        )[0]
 
     return catalogues
 
 
-async def _get_datacloud_catalogue(id_catalogue, progress):
-    """Get catalogue asynchronously. First attempts local lookup, then
-    queries SsODNet.
+async def _get_datacloud_catalogue(id_catalogue, progress_bar, progress, local):
+    """Get catalogue asynchronously. First attempt local lookup, then query SsODNet.
 
     Parameters
-    ==========
+    ----------
     id_catalogue : list
         Asteroid - catalogue combinations.
     progress : bool or tdqm.std.tqdm
         If progress is True, this is a progress bar instance. Else, it's False.
+    local : bool
+        If False, forces the remote query of the ssoCard. Default is True.
 
     Returns
-    =======
+    -------
     list of dict
         list containing len(id_) list with dictionaries corresponding to the
     catalogues of the passed identifiers. If the catalogue is not available, the dict
@@ -236,7 +229,9 @@ async def _get_datacloud_catalogue(id_catalogue, progress):
 
         tasks = [
             asyncio.ensure_future(
-                _local_or_remote_catalogue(i[0], i[1], session, progress)
+                _local_or_remote_catalogue(
+                    i[0], i[1], session, progress_bar, progress, local
+                )
             )
             for i in id_catalogue
         ]
@@ -246,28 +241,33 @@ async def _get_datacloud_catalogue(id_catalogue, progress):
     return results
 
 
-async def _local_or_remote_catalogue(id_ssodnet, catalogue, session, progress):
+async def _local_or_remote_catalogue(
+    id_ssodnet, catalogue, session, progress_bar, progress, local
+):
     """Check for presence of ssoCard in cache directory. Else, query from SsODNet."""
 
-    if progress:
-        progress.update()
-
     PATH_CATALOGUE = os.path.join(rocks.PATH_CACHE, f"{id_ssodnet}_{catalogue}.json")
 
-    if os.path.isfile(PATH_CATALOGUE):
+    if os.path.isfile(PATH_CATALOGUE) and local:
         with open(PATH_CATALOGUE, "r") as file_card:
+            progress_bar.update(progress, advance=1)
             return json.load(file_card)
 
     # Local retrieval failed, do remote query
     cat = await _query_datacloud(id_ssodnet, catalogue, session)
     cat = cat["data"][id_ssodnet]["datacloud"]
 
-    if cat:
+    # Always save the result, even if catalogue is empty
+    if cat is not None:
         cat = cat[catalogue]
-        # save to cache
-        with open(PATH_CATALOGUE, "w") as file_card:
-            json.dump(cat, file_card)
+    else:
+        cat = {}
 
+    # save to cache
+    with open(PATH_CATALOGUE, "w") as file_card:
+        json.dump(cat, file_card)
+
+    progress_bar.update(progress, advance=1)
     return cat
 
 
@@ -297,7 +297,7 @@ async def _query_datacloud(id_ssodnet, catalogue, session):
 
     if not response.ok:
         warnings.warn(f"Catalogue query failed for ID: {id_ssodnet} - {catalogue}")
-        return {}
+        return {"data": {id_ssodnet: {"datacloud": None}}}
 
     response_json = await response.json()
     return response_json
